/* Copyright (C) 2000 Red Hat, Inc.
 * Copyright (C) 2004 VMware, Inc.
 * This file is part of the Pixops Library.
 *
 * The pixops library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as published
 * by the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * The pixops library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 * pixops.h --
 *
 *      Public header for Pixops Library.
 */

#ifndef PIXOPS_H
#define PIXOPS_H

#define PIXOPS_VERSION_STR "2.0"

/* Return codes for pixops functions */
typedef enum {
        PIXOPS_STATUS_SUCCESS,
        PIXOPS_STATUS_UNSUPPORTED_FORMAT,
        PIXOPS_STATUS_FAILURE
} PixopsStatus;

/* Interpolation modes; must match GdkInterpType */
typedef enum {
	PIXOPS_INTERP_NEAREST,
	PIXOPS_INTERP_TILES,
	PIXOPS_INTERP_BILINEAR,
	PIXOPS_INTERP_HYPER
} PixopsInterpType;

/* Check if the specified source and destination formats are supported
 * by the library for compositing.
 */
typedef PixopsStatus (*pixops_support_scale_composite_formats_fn) (int, int,
                                                                   int, int);

PixopsStatus pixops_support_scale_composite_formats (int dest_channels,
                                                     int dest_has_alpha,
                                                     int src_channels,
                                                     int src_has_alpha);

/* Scale src_buf from src_width / src_height by factors scale_x, scale_y
 * and composite the portion corresponding to
 * render_x, render_y, render_width, render_height in the new
 * coordinate system into dest_buf starting at 0, 0
 */
typedef PixopsStatus (*pixops_scale_composite_fn) (unsigned char *,
                                                   int, int, int, int,
                                                   int, int, int,
                                                   const unsigned char *,
                                                   int, int, int, int, int,
                                                   double, double,
                                                   PixopsInterpType, int);

PixopsStatus pixops_scale_composite (unsigned char *dest_buf,
                                     int render_x0,
                                     int render_y0,
                                     int render_x1,
                                     int render_y1,
                                     int dest_rowstride,
                                     int dest_channels,
                                     int dest_has_alpha,
                                     const unsigned char *src_buf,
                                     int src_width,
                                     int src_height,
                                     int src_rowstride,
                                     int src_channels,
                                     int src_has_alpha,
                                     double scale_x,
                                     double scale_y,
                                     PixopsInterpType interp_type,
                                     int overall_alpha);

/* Scale src_buf from src_width / src_height by factors scale_x, scale_y
 * and composite the portion corresponding to
 * render_x, render_y, render_width, render_height in the new
 * coordinate system against a checkboard with checks of size check_size
 * of the colors color1 and color2 into dest_buf starting at 0, 0
 */
typedef PixopsStatus (*pixops_scale_composite_color_fn) (unsigned char *,
                                                         int, int, int, int,
                                                         int, int, int,
                                                         const unsigned char *,
                                                         int, int, int, int, int,
                                                         double, double,
                                                         PixopsInterpType,
                                                         int, int, int, int,
                                                         unsigned int, unsigned int);

PixopsStatus pixops_scale_composite_color (unsigned char *dest_buf,
                                           int render_x0,
                                           int render_y0,
                                           int render_x1,
                                           int render_y1,
                                           int dest_rowstride,
                                           int dest_channels,
                                           int dest_has_alpha,
                                           const unsigned char *src_buf,
                                           int src_width,
                                           int src_height,
                                           int src_rowstride,
                                           int src_channels,
                                           int src_has_alpha,
                                           double scale_x,
                                           double scale_y,
                                           PixopsInterpType interp_type,
                                           int overall_alpha,
                                           int check_x,
                                           int check_y,
                                           int check_size,
                                           unsigned int color1,
                                           unsigned int color2);


/* Check if the specified source and destination formats are supported
 * by the library for scaling.
 */
typedef PixopsStatus (*pixops_support_scale_formats_fn) (int, int, int, int, int,
                                                         unsigned int *, int);

PixopsStatus pixops_support_scale_formats (int dest_bytes_per_pixel,
                                           int dest_depth,
                                           int dest_has_alpha,
                                           int src_bytes_per_pixel,
                                           int src_depth,
                                           unsigned int *src_colormap,
                                           int src_has_alpha);

/* Scale src_buf from src_width / src_height by factors scale_x, scale_y
 * and composite the portion corresponding to
 * render_x, render_y, render_width, render_height in the new
 * coordinate system into dest_buf starting at 0, 0
 */

typedef PixopsStatus (*pixops_scale_fn) (unsigned char *,
                                         int, int, int, int,
                                         int, int, int, int,
                                         const unsigned char *,
                                         int, int, int, int, int,
                                         unsigned int *, int, double, double,
                                         PixopsInterpType);

PixopsStatus pixops_scale (unsigned char *dest_buf,
                           int render_x0,
                           int render_y0,
                           int render_x1,
                           int render_y1,
                           int dest_rowstride,
                           int dest_bytes_per_pixel,
                           int dest_depth,
                           int dest_has_alpha,
                           const unsigned char *src_buf,
                           int src_width,
                           int src_height,
                           int src_rowstride,
                           int src_bytes_per_pixel,
                           int src_depth,
                           unsigned int *src_colormap,
                           int src_has_alpha,
                           double scale_x,
                           double scale_y,
                           PixopsInterpType interp_type);
#endif
